"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.commandNames = void 0;
exports.rcOptionsTypes = rcOptionsTypes;
exports.cliOptionsTypes = cliOptionsTypes;
exports.help = help;
exports.handler = handler;
const cli_utils_1 = require("@pnpm/cli-utils");
const common_cli_options_help_1 = require("@pnpm/common-cli-options-help");
const config_1 = require("@pnpm/config");
const config_deps_installer_1 = require("@pnpm/config.deps-installer");
const error_1 = require("@pnpm/error");
const plugin_commands_env_1 = require("@pnpm/plugin-commands-env");
const store_connection_manager_1 = require("@pnpm/store-connection-manager");
const pick_1 = __importDefault(require("ramda/src/pick"));
const render_help_1 = __importDefault(require("render-help"));
const installDeps_1 = require("./installDeps");
const config_config_writer_1 = require("@pnpm/config.config-writer");
function rcOptionsTypes() {
    return (0, pick_1.default)([
        'cache-dir',
        'child-concurrency',
        'dangerously-allow-all-builds',
        'engine-strict',
        'fetch-retries',
        'fetch-retry-factor',
        'fetch-retry-maxtimeout',
        'fetch-retry-mintimeout',
        'fetch-timeout',
        'force',
        'global-bin-dir',
        'global-dir',
        'global-pnpmfile',
        'global',
        'hoist',
        'hoist-pattern',
        'https-proxy',
        'ignore-pnpmfile',
        'ignore-scripts',
        'ignore-workspace-root-check',
        'link-workspace-packages',
        'lockfile-dir',
        'lockfile-directory',
        'lockfile-only',
        'lockfile',
        'modules-dir',
        'network-concurrency',
        'node-linker',
        'noproxy',
        'npm-path',
        'package-import-method',
        'pnpmfile',
        'prefer-offline',
        'production',
        'proxy',
        'public-hoist-pattern',
        'registry',
        'reporter',
        'save-dev',
        'save-exact',
        'save-optional',
        'save-peer',
        'save-prefix',
        'save-prod',
        'save-workspace-protocol',
        'shamefully-flatten',
        'shamefully-hoist',
        'shared-workspace-lockfile',
        'side-effects-cache-readonly',
        'side-effects-cache',
        'store-dir',
        'strict-peer-dependencies',
        'unsafe-perm',
        'offline',
        'only',
        'optional',
        'use-running-store-server',
        'use-store-server',
        'verify-store-integrity',
        'virtual-store-dir',
    ], config_1.types);
}
function cliOptionsTypes() {
    return {
        ...rcOptionsTypes(),
        'allow-build': [String, Array],
        recursive: Boolean,
        save: Boolean,
        workspace: Boolean,
        config: Boolean,
    };
}
exports.commandNames = ['add'];
function help() {
    return (0, render_help_1.default)({
        description: 'Installs a package and any packages that it depends on.',
        descriptionLists: [
            {
                title: 'Options',
                list: [
                    {
                        description: 'Save package to your `dependencies`. The default behavior',
                        name: '--save-prod',
                        shortAlias: '-P',
                    },
                    {
                        description: 'Save package to your `devDependencies`',
                        name: '--save-dev',
                        shortAlias: '-D',
                    },
                    {
                        description: 'Save package to your `optionalDependencies`',
                        name: '--save-optional',
                        shortAlias: '-O',
                    },
                    {
                        description: 'Save package to your `peerDependencies` and `devDependencies`',
                        name: '--save-peer',
                    },
                    {
                        description: 'Install exact version',
                        name: '--[no-]save-exact',
                        shortAlias: '-E',
                    },
                    {
                        description: 'Save packages from the workspace with a "workspace:" protocol. True by default',
                        name: '--[no-]save-workspace-protocol',
                    },
                    {
                        description: 'Install as a global package',
                        name: '--global',
                        shortAlias: '-g',
                    },
                    {
                        description: 'Run installation recursively in every package found in subdirectories \
or in every workspace package, when executed inside a workspace. \
For options that may be used with `-r`, see "pnpm help recursive"',
                        name: '--recursive',
                        shortAlias: '-r',
                    },
                    {
                        description: 'Only adds the new dependency if it is found in the workspace',
                        name: '--workspace',
                    },
                    {
                        description: 'Save the dependency to configurational dependencies',
                        name: '--config',
                    },
                    common_cli_options_help_1.OPTIONS.ignoreScripts,
                    common_cli_options_help_1.OPTIONS.offline,
                    common_cli_options_help_1.OPTIONS.preferOffline,
                    common_cli_options_help_1.OPTIONS.storeDir,
                    common_cli_options_help_1.OPTIONS.virtualStoreDir,
                    common_cli_options_help_1.OPTIONS.globalDir,
                    ...common_cli_options_help_1.UNIVERSAL_OPTIONS,
                    {
                        description: 'A list of package names that are allowed to run postinstall scripts during installation',
                        name: '--allow-build',
                    },
                ],
            },
            common_cli_options_help_1.FILTERING,
        ],
        url: (0, cli_utils_1.docsUrl)('add'),
        usages: [
            'pnpm add <name>',
            'pnpm add <name>@<tag>',
            'pnpm add <name>@<version>',
            'pnpm add <name>@<version range>',
            'pnpm add <git host>:<git user>/<repo name>',
            'pnpm add <git repo url>',
            'pnpm add <tarball file>',
            'pnpm add <tarball url>',
            'pnpm add <dir>',
        ],
    });
}
async function handler(opts, params) {
    if (opts.cliOptions['save'] === false) {
        throw new error_1.PnpmError('OPTION_NOT_SUPPORTED', 'The "add" command currently does not support the no-save option');
    }
    if (!params || (params.length === 0)) {
        throw new error_1.PnpmError('MISSING_PACKAGE_NAME', '`pnpm add` requires the package name');
    }
    if (opts.config) {
        const store = await (0, store_connection_manager_1.createOrConnectStoreController)(opts);
        await (0, config_deps_installer_1.resolveConfigDeps)(params, {
            ...opts,
            store: store.ctrl,
            rootDir: opts.workspaceDir ?? opts.rootProjectManifestDir,
        });
        return;
    }
    if (!opts.recursive &&
        opts.workspaceDir === opts.dir &&
        !opts.ignoreWorkspaceRootCheck &&
        !opts.workspaceRoot &&
        opts.workspacePackagePatterns &&
        opts.workspacePackagePatterns.length > 1) {
        throw new error_1.PnpmError('ADDING_TO_ROOT', 'Running this command will add the dependency to the workspace root, ' +
            'which might not be what you want - if you really meant it, ' +
            'make it explicit by running this command again with the -w flag (or --workspace-root). ' +
            'If you don\'t want to see this warning anymore, you may set the ignore-workspace-root-check setting to true.');
    }
    if (opts.global) {
        if (!opts.bin) {
            throw new error_1.PnpmError('NO_GLOBAL_BIN_DIR', 'Unable to find the global bin directory', {
                hint: 'Run "pnpm setup" to create it automatically, or set the global-bin-dir setting, or the PNPM_HOME env variable. The global bin directory should be in the PATH.',
            });
        }
        if (params.includes('pnpm') || params.includes('@pnpm/exe')) {
            throw new error_1.PnpmError('GLOBAL_PNPM_INSTALL', 'Use the "pnpm self-update" command to install or update pnpm');
        }
    }
    const include = {
        dependencies: opts.production !== false,
        devDependencies: opts.dev !== false,
        optionalDependencies: opts.optional !== false,
    };
    if (opts.allowBuild?.length) {
        if (opts.argv.original.includes('--allow-build')) {
            throw new error_1.PnpmError('ALLOW_BUILD_MISSING_PACKAGE', 'The --allow-build flag is missing a package name. Please specify the package name(s) that are allowed to run installation scripts.');
        }
        if (opts.rootProjectManifest?.pnpm?.ignoredBuiltDependencies?.length) {
            const overlapDependencies = opts.rootProjectManifest.pnpm.ignoredBuiltDependencies.filter((dep) => opts.allowBuild?.includes(dep));
            if (overlapDependencies.length) {
                throw new error_1.PnpmError('OVERRIDING_IGNORED_BUILT_DEPENDENCIES', `The following dependencies are ignored by the root project, but are allowed to be built by the current command: ${overlapDependencies.join(', ')}`, {
                    hint: 'If you are sure you want to allow those dependencies to run installation scripts, remove them from the pnpm.ignoredBuiltDependencies list.',
                });
            }
        }
        opts.onlyBuiltDependencies = Array.from(new Set([
            ...(opts.onlyBuiltDependencies ?? []),
            ...opts.allowBuild,
        ])).sort((a, b) => a.localeCompare(b));
        if (opts.rootProjectManifestDir) {
            opts.rootProjectManifest = opts.rootProjectManifest ?? {};
            await (0, config_config_writer_1.writeSettings)({
                ...opts,
                workspaceDir: opts.workspaceDir ?? opts.rootProjectManifestDir,
                updatedSettings: {
                    onlyBuiltDependencies: opts.onlyBuiltDependencies,
                },
            });
        }
    }
    return (0, installDeps_1.installDeps)({
        ...opts,
        include,
        includeDirect: include,
        prepareExecutionEnv: plugin_commands_env_1.prepareExecutionEnv.bind(null, opts),
    }, params);
}
//# sourceMappingURL=add.js.map